from flask import Flask


__author__ = 'slippers'


class ProseTestCase():

    def setUp(self):
        self.app = Flask(__name__)
        self.app.config['SECRET_KEY'] = "test-secret"
        self.app.config["WTF_CSRF_ENABLED"] = False  # to bypass CSRF token
        self.client = self.app.test_client()

        @self.app.route("/")
        def index():
            return "Hello World!"

    def _create_storage(self):
        raise NotImplementedError("Subclass must implement abstract method")

    def tearDown(self):
        raise NotImplementedError("Subclass must implement abstract method")



import unittest
import pytest
import random
import uuid
try:
    from builtins import range
except ImportError:
    pass
import unittest
import os
from .prosetestcase import ProseTestCase
import time

class StorageTestMethods(ProseTestCase):

    def setUp(self):
        ProseTestCase.setUp(self)
        self._create_storage()

    def test_new_corpora(self):
        with pytest.raises(Exception) as excinfo:
            corpora = self.storage.corpora_save(label='test', text='hoorah')
        assert str(excinfo.value).startswith('MarkovText not found for corpora:')

    def test_prose(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora1 = self.storage.corpora_save(label='data1', text=data)

            print('corpora1:', corpora1)

            self.assertIsNotNone(corpora1)
            prose1 = self.storage.prose()

            self.assertIsNotNone(prose1)
            print('prose1:', prose1)
            assert(prose1['id'])
            assert(prose1['corporas'])
            assert(prose1['prosetype'])

            prose2 = self.storage.prose(corpora=set().add(corpora1))
            self.assertIsNotNone(prose2)
            prose3 = self.storage.prose(uuid=prose1['id'])
            self.assertIsNotNone(prose3)

    def test_corpora_list(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora1 = self.storage.corpora_save(label='data1', text=data)
            self.assertIsNotNone(corpora1)
            corpora2 = self.storage.corpora_save(label='data2', text=data)
            self.assertIsNotNone(corpora2)
            clist = self.storage.corpora_list()
            assert(len(clist) == 2)
            clist1 = self.storage.corpora_list(uuid=corpora1)
            assert(len(clist1) == 1)

    def test_corpora_generate_prose(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora = self.storage.corpora_save(label='data', text=data)
            corpora_clone = self.storage.corpora_save(label='data clone', text=data)
            self.assertIsNotNone(corpora)
            self.storage.corpora_generate_prose(set([corpora]))
            p = self.storage.prose()
            print('prose:', p)
            self.assertIsNotNone(p)

            cp = self.storage.prose(corpora=corpora)
            print('cp:', corpora, cp)

    def test_multiple_corpora_prose(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora1 = self.storage.corpora_save(label='data1', text=data)
            self.assertIsNotNone(corpora1)
            corpora2 = self.storage.corpora_save(label='data2', text=data)
            self.assertIsNotNone(corpora2)
            corpora3 = self.storage.corpora_save(label='data3', text=data)
            self.assertIsNotNone(corpora3)
            corpora4 = self.storage.corpora_save(label='data4', text=data)
            self.assertIsNotNone(corpora4)

            corpora_set = set([corpora1, corpora2, corpora3, corpora4])

            self.storage.corpora_generate_prose(corpora_set)
            p = self.storage.prose()
            self.assertIsNotNone(p)

    def test_rating(self):

        xray = self.storage.ratings()
        xray = self.storage.ratings(prose_id=uuid.uuid1())

        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora1 = self.storage.corpora_save(label='data1', text=data)
            self.assertIsNotNone(corpora1)

            prose1 = self.storage.prose()
            prose2 = self.storage.prose()
            prose3 = self.storage.prose()

            # create saw grock
            for x in range(100):
                self.storage.prose()

            self.storage.grock(prose_id=prose3['id'], reaction='meh')
            self.storage.grock(prose_id=prose3['id'], reaction='omg')
            self.storage.grock(prose_id=prose3['id'], reaction='omg')

            self.storage.grock(prose_id=prose1['id'], reaction='meh')
            self.storage.grock(prose_id=prose1['id'], reaction='meh')

            self.storage.grock(prose_id=prose2['id'], reaction='omg')
            self.storage.grock(prose_id=prose2['id'], reaction='omg')

            xray = self.storage.ratings()

            delta = self.storage.ratings(prose_id=prose3['id'])

            assert(len(delta) == 1)

            meh = self.storage.ratings(rate_type='meh')


    def test_grock_rating(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora1 = self.storage.corpora_save(label='data1', text=data)
            self.assertIsNotNone(corpora1)

            prose1 = self.storage.prose()
            prose2 = self.storage.prose()

            # create saw grock
            for x in range(100):
                self.storage.prose()

            for x in range(20):
                self.storage.grock(prose_id=prose1['id'], reaction='omg')
                self.storage.grock(prose_id=prose2['id'], reaction='omg')

            for x in range(10):
                self.storage.grock(prose_id=prose1['id'], reaction='meh')
                self.storage.grock(prose_id=prose2['id'], reaction='meh')

            xray = self.storage.ratings()

            assert(len(xray) == 2)


    def test_grock_rating_rand(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            corpora1 = self.storage.corpora_save(label='data1', text=data)
            self.assertIsNotNone(corpora1)

            for x in range(1000):
                reaction = random.choice(['omg','meh'])
                p = self.storage.prose()
                self.storage.grock(prose_id=p['id'], reaction=reaction)

            xray = self.storage.ratings(limit=50)



import unittest
import os
import json
from flask import Flask, session
from flask_sqlalchemy import SQLAlchemy
from flask_prose import FSQLAStorage, ProseEngine
from flask_jsontools import FlaskJsonClient, JsonResponse
from flask_security import (
    Security,
    auth_token_required,
    roles_required
)
from .security_models import SetupModels, SetupUsers

class ApiTest(unittest.TestCase):

    def setUp(self):
        print('setUp')
        self.app = app = Flask(__name__)
        self.app.test_client_class = FlaskJsonClient
        self.app.debug = self.app.testing = True
        self.app.config['SECRET_KEY'] = os.urandom(24)

        #config sqlalchemy
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
        app.config['DATABASE_URI'] = 'sqlite://'
        app.config['SQLALCHEMY_BINDS'] = {'prose':'sqlite://'}
        self.db = db = SQLAlchemy(app)

        #config flask-security
        app.config['WTF_CSRF_ENABLED'] = False
        user_datastore = SetupModels(db)
        security = Security(app, user_datastore)

        # config flask-prose
        self.storage = FSQLAStorage(db=db, bind_key='prose')
        self.prose = ProseEngine()
        self.prose.viewmethod_decorators(
            corpora=(auth_token_required, roles_required('prose_admin'),)
        )
        self.prose.init_app(app=app, storage=self.storage)

        self.xray = os.path.join(os.path.dirname(__file__), 'little_shak')

        with open(self.xray, 'r') as myfile:
            self.storage.corpora_save(label='setup', text=myfile.read())


        #print(app.url_map)

        @app.before_first_request
        def create_user():
            self.db.create_all()
            SetupUsers(user_datastore)
            self.db.session.commit()

    def getAuthToken(self, test_client, user):
        """
            flask-security contains the /login endpoint
            that when posed to will return the json auth_token

            the flask-security @auth_token_required decorator
            will do the method access checking

            must set this config for token authentication to work.
            app.config['WTF_CSRF_ENABLED'] = False
        """
        user = {'email':user, 'password':'test123'}
        ru = test_client.post('/login', user)
        ru_json = ru.get_json()
        print('getAuthToken:token', ru_json)
        return {'authentication-token': ru_json['response']['user']['authentication_token']}

    def test_upload(self):
        with open(self.xray, 'rb') as myfile:
            with self.app.test_client() as c:
                auth_token = self.getAuthToken(c, 'test@example.com')
                rv = c.post('/v1/corpora',
                            data = {'file': (myfile,'shatit_now')},
                            headers=auth_token)

                self.assertEqual(rv.status_code, 200)

                rv_dict = json.loads(rv.get_json())

                rl = c.get('/v1/corpora/{0}'.format(rv_dict[0]['id']),
                           headers=auth_token)
                self.assertEqual(rl.status_code, 200)
                self.assertIsInstance(rl, JsonResponse)
                print(rl.get_json())

                rp = c.get('/v1/prose')
                print(rp.get_json())

    def test_no_token(self):
        with self.app.test_client() as c:
            rv = c.get('/v1/prose')
            print(rv.get_json())

    def test_corpora_list(self):
        with self.app.test_client() as c:
            auth_token = self.getAuthToken(c, 'test@example.com')

            rv = c.get('/v1/corpora', headers=auth_token)
            print(rv)
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)

    def test_prose(self):
         with self.app.test_client() as c:
            rv = c.get('/v1/prose')
            print(rv.get_json())
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)



import unittest
import random
import uuid
try:
    from builtins import range
except ImportError:
    pass
import unittest
import os
from .prosetestcase import ProseTestCase
import time



class StorageTestTables(ProseTestCase):

    def setUp(self):
        ProseTestCase.setUp(self)
        self._create_storage()

    def check_table(self, table_name, expected_columns):
        table = self.metadata.tables[table_name]
        columns = [t.name for t in table.columns]
        self.assertListEqual(columns, expected_columns)

    def test_corpora_table_exists(self):
        table_name = "corpora"
        expected_columns = ['id', 'label', 'source', 'text', 'post_date']
        self.check_table(table_name, expected_columns)

    def test_prose_table_exists(self):
        table_name = "prose"
        expected_columns = ['id', 'prosetype_id', 'text']
        self.check_table(table_name, expected_columns)

    def test_prosetype_table_exists(self):
        table_name = "prosetype"
        expected_columns = ['id', 'label']
        self.check_table(table_name, expected_columns)

    def test_grock_table_exists(self):
        table_name = "grock"
        expected_columns = ['id', 'prose_id', 'reaction', 'created_date']
        self.check_table(table_name, expected_columns)

    def test_prosecorpora_table_exists(self):
        table_name = "prosecorpora"
        expected_columns = ['prose_id', 'corpora_id']
        self.check_table(table_name, expected_columns)




from flask_security import (
    SQLAlchemyUserDatastore,
    UserMixin,
    RoleMixin,
)
from sqlalchemy.ext.declarative import (
    declarative_base,
    declared_attr,
    as_declarative
)
from sqlalchemy.orm import relationship, backref
from sqlalchemy import (
    Table,
    Column,
    Integer,
    String,
    Boolean,
    ForeignKey,
    Text,
    DateTime,
    SmallInteger,
    PrimaryKeyConstraint
)


def SetupModels(db):
    # db is a flask_sqlalchemy instance

    Base = db.Model

    # Define models
    class RoleUsers(Base):
        __tablename__ = 'roles_users'
        id = Column(Integer(), primary_key=True)
        user_id = Column(Integer(), ForeignKey('users.id'))
        role_id = Column(Integer(), ForeignKey('roles.id'))


    class Role(Base, RoleMixin):
        __tablename__ = 'roles'
        id = Column(Integer(), primary_key=True)
        name = Column(String(80), unique=True)
        description = Column(String(255))


    class User(Base, UserMixin):
        __tablename__ = 'users'
        id = Column(Integer, primary_key=True)
        email = Column(String(255), unique=True)
        password = Column(String(255))
        active = Column(Boolean())
        confirmed_at = Column(DateTime())
        roles = relationship('Role',
                             secondary='roles_users',
                             backref=backref('users', lazy='dynamic'))

    return SQLAlchemyUserDatastore(db, User, Role)


def SetupUsers(user_datastore):
    user_datastore.find_or_create_role(name='prose_admin', description='prose administrator')
    user_datastore.find_or_create_role(name='reader', description='prose reader')

    if not user_datastore.get_user('reader@example.com'):
        user_datastore.create_user(email='reader@example.com', password='test123')
    user_datastore.add_role_to_user('reader@example.com', 'reader')

    if not user_datastore.get_user('test@example.com'):
        user_datastore.create_user(email='test@example.com', password='test123')
    user_datastore.add_role_to_user('test@example.com', 'prose_admin')



import re
try:
    from builtins import str
except ImportError:
    pass
import os.path
import logging
import json
import uuid
from flask_jsontools import DynamicJSONEncoder
from sqlalchemy.ext.serializer import loads, dumps
from sqlalchemy import (
    select,
    desc,
    func,
    and_,
    not_,
)
from sqlalchemy.orm import (
    sessionmaker,
    load_only,
    aliased,
    subqueryload,
    joinedload
)
from sqlalchemy.exc import IntegrityError
from .models import (
    Base,
    Prose,
    Grock,
    ProseType,
    Corpora,
    MarkovText,
    ProseCorpora
)
from .prosemaker import (
    ProseMakerText,
    ProseMakerSen
)


class Storage():

    def __init__(self, engine=None, bind_key=None):

        self._logger = logging.getLogger(__name__)

        self.generate_prose_callback = self.corpora_generate_prose
        self.generate_markov_callback = self.corpora_generate_markov

        if engine is None:
            raise ValueError('engine is required')

        self._engine = engine

        # __bind_key__ is a custom attribute set in the model
        # it is used by wrapper extentions like flask-sqlalchemy and flask-alchy
        # to bind the model to a engine connection
        if bind_key:
            Base.__bind_key__ = bind_key

        Session = sessionmaker(bind=engine)

        self._session = Session()

        # the models have inherited Base, we have imported base from there.
        Base.metadata.create_all(engine)

    def close(self):
        '''
        each time session.commit() is called an implict tansaction is newly created.
        this sqlalchemy behavior causes an issue with issuing DDL
        commands in newer versions of mysql.
        error 'Waiting for table metadata lock'
        '''
        self._session.close()

    def prosetype_add(self, label):
        if not label:
            raise ValueError('label requried')
        pt = self._session.query(ProseType) \
                    .filter(ProseType.label == label) \
                    .one_or_none()

        if pt:
            return pt.id

        try:
            pt = ProseType(label)
            self._session.add(pt)
            self._session.commit()
            return pt.id
        except:
           self._session.rollback()
           raise

    def corpora_save(self, label=None, source=None, text=None):
        rx = re.compile('[^\w\s\.\,\?\!\'\"]')
        res = rx.sub(' ', text).strip()
        if not res:
            raise ValueError('text invalid.')
        self._logger.debug('label:%s text:%s...', label, res[0:20])
        corpora = Corpora(text=res, label=label, source=source)
        try:
            self._session.add(corpora)
            self._session.commit()
            self._logger.debug('corpora.id:%s', corpora.id)
        except:
           self._session.rollback()
           raise

        if self.generate_markov_callback:
            self._logger.debug('calling generate_markov_callback:%s',
                               self.generate_markov_callback)
            self.generate_markov_callback(corpora.id)

        if self.generate_prose_callback:
            self._logger.debug('calling generate_prose_callback:%s',
                               self.generate_prose_callback)
            self.generate_prose_callback((corpora.id,))

        return corpora.id

    def corpora_list(self, uuid=None):
        corpora = self._session.query( \
                                      Corpora.id,
                                      Corpora.label,
                                      Corpora.post_date,
                                      Corpora.source,
                                      func.count(ProseCorpora.prose_id).label('prose'),
                                     ) \
                .select_from(Corpora) \
                .join(ProseCorpora, ProseCorpora.corpora_id == Corpora.id) \
                .group_by(Corpora.id)

        if uuid:
            corpora = corpora.filter(Corpora.id == uuid)

        self._logger.debug('corpora_list:%s', str(corpora))
        corpora_result  = corpora.all()
        if corpora_result:
            return json.loads(json.dumps(corpora_result, cls=DynamicJSONEncoder))
        else:
            return []

    def corpora_delete(self, uuid):
        try:
            corpora = self._session.query(Corpora) \
                    .options(load_only('id')) \
                    .filter(Corpora.id == uuid).one_or_none()

            if corpora:
                self._session.delete(corpora)
                self._session.commit()
                return True

            return False
        except:
            self._session.rollback()
            raise

    def generate_markov(self, callback):
        self.generate_markov_callback = callback
        return callback

    def corpora_generate_markov(self, corpora_id):
        """
        create markov sentences from corpora text
        """
        try:
            corpora = self._session.query(Corpora) \
                    .filter(Corpora.id == corpora_id) \
                    .one_or_none()

            if not corpora:
                raise Exception('corpora not found')

            pm = ProseMakerText(text=corpora.text)
            markovtext = [MarkovText(corpora.id, sen) for sen in pm.get_sentences()]
            self._logger.debug('markovtext count:%s',len(markovtext))
            self._session.bulk_save_objects(markovtext)
            self._session.commit()
        except:
            self._session.rollback()
            raise

    def generate_prose(self, callback):
        self.generate_prose_callback = callback
        return callback

    def corpora_generate_prose(self, corpora=set()):
        """
        arg:corpora = set of corpora ids
        """
        MAX_SENTENCE_COUNT = 1000

        corpora_result = self._session.query(Corpora) \
                .filter(Corpora.id.in_(list(corpora))) \
                .all()

        corpora_ids = [str(x.id) for x in corpora_result]

        self._logger.debug('corpora_ids:%s',corpora_ids)

        mtext = self._session.query(MarkovText) \
                .filter(MarkovText.corpora_id.in_(corpora_ids)) \
                .order_by(func.random()) \
                .limit(MAX_SENTENCE_COUNT) \
                .all()

        self._logger.debug('mtext:%s',len(mtext))
        if not mtext:
            raise Exception('MarkovText not found for corpora:{}'.format(corpora_ids))

        # get the json of the markovtext model
        pm = ProseMakerSen([m.json() for m in mtext])

        self.insert_prose(pm, 'stanza')
        self.insert_prose(pm, 'haiku')

    def insert_prose(self,
                     prosemaker,
                     prose_type,
                     MAX_PROSE_COUNT = 5):
        prosetype_id = self.prosetype_add(prose_type)

        for x in range(MAX_PROSE_COUNT):
            prose_json = prosemaker.get_prose(prose_type)

            self._logger.debug('prose_json:%s', prose_json)

            if not prose_json:
                raise Exception('failed to generate prose')

            try:
                prose = Prose(prosetype_id=prosetype_id, text=prose_json)

                self._session.add(prose)
                self._session.commit()
                self._logger.debug('prose %s:%s', prose_type, prose.id)

                # gather unique corpora ids here
                corpora_ids = {(x['corpora_id']) for x in prose_json}

                # create the association ProseCorpora objects
                prose_corpora = []
                prose_corpora.extend([ProseCorpora(prose.id, c_id) for c_id in corpora_ids])
                self._session.bulk_save_objects(prose_corpora)
                self._session.commit()
            except:
                self._session.rollback()
                raise

    def prose(self, uuid=None, corpora=()):
        self._logger.debug('uuid:%s corpora:%s', uuid, corpora)
        prose = None
        if uuid:
            prose = self._prose(uuid)
        elif corpora:
            prose = self._prose_corpora_random(corpora)
        else:
            prose = self._prose_random()
        return self._prose_data(prose)

    def _prose(self, uuid):
        if not uuid:
            raise ValueError('uuid required.')
        prose = self._session.query(Prose.id) \
                .filter(Prose.id == uuid) \
                .first()

        return prose

    def _prose_random(self):
        prose = self._session.query(Prose.id) \
                .order_by(func.random()) \
                .limit(1) \
                .first()

        return prose

    def _prose_corpora_random(self, corpora):
        if not corpora:
            raise ValueError('corpora required.')

        prose = self._session.query(Prose.id) \
                .join(ProseCorpora, ProseCorpora.prose_id == Prose.id) \
                .filter(ProseCorpora.corpora_id == corpora) \
                .order_by(func.random()) \
                .limit(1) \
                .first()

        return prose

    def _prose_data(self, prose_id):
        if not prose_id:
            return {}

        prose = self._session.query(Prose) \
                .filter(Prose.id == prose_id[0]) \
                .first()

        if not prose:
            raise Exception('prose not found.')

        prosetype = self._session.query(ProseType) \
                .filter(prose.prosetype_id == ProseType.id) \
                .first()

        corpora = self._session.query(Corpora) \
                .join(ProseCorpora, ProseCorpora.corpora_id == Corpora.id) \
                .filter(ProseCorpora.prose_id == prose.id) \
                .all()

        prose_data = json.loads(json.dumps(prose, cls=DynamicJSONEncoder))
        prose_data['prosetype'] = json.loads(json.dumps(prosetype, cls=DynamicJSONEncoder))
        prose_data['corporas'] = json.loads(json.dumps(corpora, cls=DynamicJSONEncoder))

        self._grock(prose.id, reaction='saw')

        return prose_data

    def _grock(self, prose_id, reaction='saw'):
        self._logger.debug('prose_id:%s reaction:%s', prose_id, reaction)
        reactions = ['omg', 'meh', 'saw']
        if reaction not in reactions:
            raise ValueError("Expected one of: %s" % reactions)

        try:
            self._session.add(Grock(prose_id, reaction))
            self._session.commit()
        except:
           self._session.rollback()
           raise

    def grock(self, prose_id, reaction='saw'):
        """
        """
        prose = self._prose(prose_id)
        if not prose:
            raise Exception('prose not found.')

        self._grock(prose.id, reaction)

    def ratings(self, rate_type='omg', prose_id=None, limit=10):
        """
        arg: rate_type = omg | meh
        controls if the query looks for meh or omg results

        arg: prose_id: just returns result for this prose id

        arg: limit: always applied defaulting to 10

        select prose_id, saw,omg,meh
        from (
        select prose_id,
        count(*) as saw,
        omg - meh as total,
        (
        select count(*)
        from grock omg
        where  omg.reaction = 'omg'
        and omg.prose_id = grock.prose_id
        ) as omg,
        (
        select count(*)
        from grock meh where  meh.reaction = 'meh'
        and meh.prose_id = grock.prose_id
        ) as meh
        from grock
        where reaction = 'saw'
        group by(prose_id)
        )
        where omg - meh > 0
        order by (omg - meh) desc, saw desc;
        """
        omg = aliased(Grock)
        meh = aliased(Grock)
        saw = aliased(Grock)

        _query = select([saw.prose_id,
                         func.count().label('saw'),

                         select([func.count()]) \
                         .select_from(meh) \
                         .where(meh.reaction == 'meh') \
                         .where(meh.prose_id == saw.prose_id) \
                         .as_scalar() \
                         .label('meh'),

                         select([func.count()]) \
                         .select_from(omg) \
                         .where(omg.reaction == 'omg') \
                         .where(omg.prose_id == saw.prose_id) \
                         .as_scalar() \
                         .label('omg'),
                        ]) \
                .select_from(saw) \
                .where(saw.reaction == 'saw') \
                .group_by(saw.prose_id)

        # if asking for a single prose constrain in subquery
        if prose_id:
            _query = _query.where(saw.prose_id == prose_id)

        # postgres requires subquery AS to be valid
        _query = _query.alias('xray')

        query = select([_query, (_query.c.omg - _query.c.meh).label('total')])

        # if asking for a single prose always return no constraint
        if not prose_id and rate_type == 'omg':
            query = query.where((_query.c.omg - _query.c.meh) > 0) \
                    .order_by((_query.c.omg - _query.c.meh).desc())

        if not prose_id and rate_type == 'meh':
            query = query.where((_query.c.omg - _query.c.meh) < 0) \
                    .order_by((_query.c.omg - _query.c.meh))

        query = query.limit(limit) \
                .alias('gamma')

        self._logger.debug('query:%s %s', prose_id, str(query))
        try:
            rating_result = self._session.query(query).all()
            if rating_result:
                return json.loads(json.dumps(rating_result, cls=DynamicJSONEncoder))
            else:
                return []
        except:
           self._session.rollback()
           raise

    def ratings_all(self):
        omg = self.ratings(rate_type='omg')
        meh = self.ratings(rate_type='meh')
        omg.extend(meh)
        return omg



import datetime
import uuid
from sqlalchemy.ext.declarative import (
    declarative_base,
    declared_attr
)
from sqlalchemy import (
    Table,
    Column,
    Integer,
    String,
    ForeignKey,
    Text,
    DateTime,
    SmallInteger,
    PrimaryKeyConstraint
)
from sqlalchemy.orm import (
    relationship,
    deferred,
    backref
)
from sqlalchemy_utils import UUIDType, JSONType
from flask_jsontools import JsonSerializableBase


class BaseModel(object):

    __bind_key__ = 'DEFAULT'
    #id = Column(UUIDType(binary=False), default=uuid.uuid4, primary_key=True)

    @declared_attr
    def __tablename__(cls):
        return cls.__name__.lower()


Base = declarative_base(cls=(BaseModel, JsonSerializableBase))


class Corpora(Base):
    id = Column(UUIDType(binary=False), default=uuid.uuid4, primary_key=True)
    # exclude text from json output,  too big.
    _json_exclude = ['text']
    # a short description of the content
    label = Column(String(256), default="")
    # source of the content, url file...
    source = Column(String(256), default="")
    text = deferred(Column(Text))
    post_date = Column(DateTime)

    def __init__(self, label, source, text):
        self.post_date = datetime.datetime.utcnow()
        self.label = label
        self.text = text
        self.source = source


class MarkovText(Base):
    id = Column(UUIDType(binary=False), default=uuid.uuid4, primary_key=True)
    corpora_id = Column(
        UUIDType(binary=False),
        ForeignKey('corpora.id', ondelete='CASCADE')
    )
    text = Column(String(500))

    def __init__(self, corpora_id, text):
        self.corpora_id = corpora_id
        self.text = text

    def json(self):
        return {
            'id':str(self.id),
            'corpora_id':str(self.corpora_id),
            'text':self.text
        }


class ProseType(Base):
    id = Column(UUIDType(binary=False), default=uuid.uuid4, primary_key=True)
    label = Column(String(256))

    def __init__(self, label):
        self.label = label


class ProseCorpora(Base):
    prose_id = Column(UUIDType(binary=False),
                      ForeignKey('prose.id'),
                      primary_key=True)
    corpora_id = Column(UUIDType(binary=False),
                        ForeignKey('corpora.id'),
                        primary_key=True)
    corpora = relationship('Corpora', lazy='joined')

    def __init__(self, prose_id, corpora_id):
        self.prose_id = prose_id
        self.corpora_id = corpora_id


class Prose(Base):
    _json_exclude = ['prosetype_id']
    id = Column(UUIDType(binary=False), default=uuid.uuid4, primary_key=True)
    prosetype_id = Column(
        UUIDType(binary=False),
        ForeignKey('prosetype.id', ondelete='CASCADE')
    )
    text = Column(JSONType)

    def __init__(self, prosetype_id, text):
        self.prosetype_id = prosetype_id
        self.text = text


class Grock(Base):
    id = Column(UUIDType(binary=False), default=uuid.uuid4, primary_key=True)
    prose_id = Column(
        UUIDType(binary=False),
        ForeignKey('prose.id', ondelete='CASCADE')
    )
    reaction = Column(String(10))
    created_date = Column(DateTime)

    def __init__(self, prose_id, reaction):
        self.prose_id = prose_id
        self.reaction = reaction
        self.created_date = datetime.datetime.utcnow()



from .engine import ProseEngine
from .prosemaker import ProseMakerText, ProseMakerSen
from .storage import Storage
from .fsqla_storage import FSQLAStorage

__version__ = '0.1.51'
__author__ = 'kirk erickson'



import random
import logging
import markovify
from textstat.textstat import textstat
from .models import MarkovText


class ProseMaker():

    def __init__(self):
        self._logger = logging.getLogger(__name__)
        self.sentences = []

    def get_prose(self, name):
        return getattr(self, name)()

    def get_sentences(self):
        return [s['text'] for s in self.sentences]

    def syllables(self, sentence):
        syllables = 0
        for word in sentence.split():
           syllables += round(textstat.syllable_count(word))
        return syllables

    def stanza(self):
        return random.sample(self.sentences, 5)

    def haiku(self):
        # 5/7/5 syllable count

        five = [d for d in self.sentences if d['syl'] == 5]

        if len(five) < 2:
            raise Exception('haiku five syllables not found.')

        seven = [d for d in self.sentences if d['syl'] == 7]

        if len(seven) < 1:
            raise Exception('haiku seven syllables not found.')

        return [random.choice(five),
                random.choice(seven),
                random.choice(five)]


class ProseMakerSen(ProseMaker):

    def __init__(self, sendict):
        ProseMaker.__init__(self)
        # arg: sendict = [{'id':id,'text':text},]
        for i in range(len(sendict)):
            self.sentences.append(
                self.analyze_dict(sendict[i])
            )

    def analyze_dict(self, sendict):
        # update the dict with len and syllables
        sendict['len'] = len(sendict['text'])
        sendict['syl'] = self.syllables(sendict['text'])
        return sendict



class ProseMakerText(ProseMaker):

    def __init__(self, text):
        ProseMaker.__init__(self)
        self.generate(text, max_sen=2000)

    def analyze_text(self, id, text):
        return {'id':id,
                'text':text,
                'len':len(text),
                'syl':self.syllables(text)}


    def generate(self, text, max_sen=1000):
        """
        Take text and convert into a
        list of sentences.
        """
        model = markovify.Text(text)
        count = 1
        tries = 5
        while count <= max_sen and tries > 0:
            sen = model.make_short_sentence(100)
            if sen:
                count += 1
                self.sentences.append(
                    self.analyze_text(count, sen)
                )
            else:
                tries -= 1
                self._logger.debug('sen count:%s tries:%s', count, tries)



from .storage import Storage


class FSQLAStorage(Storage):
    """flask_sqlalchemy wrapper class for SQLAStorage
    """
    def __init__(self, db=None, bind_key=None):
        """
        :param db is flask_sqlalchemy SQLAlchemy object
        :param bind_key: (Optional) Reference the database to bind for multiple
        database scenario with binds.
        """
        if db is None:
            raise ValueError("db cannot be None.")

        engine = db.get_engine(db.get_app(), bind=bind_key)

        Storage.__init__(self, engine=engine, bind_key=bind_key)

        db.metadata.reflect(engine)



import unittest
from .storage_test_tables import StorageTestTables
from .storage_test_methods import StorageTestMethods
try:
    from builtins import range
except ImportError:
    pass
import tempfile
import os
from sqlalchemy import create_engine, MetaData
import time
from flask_prose import Storage


class StorageTest():

    def _create_storage(self):
        temp_dir = tempfile.gettempdir()
        self._dbfile = os.path.join(temp_dir, "temp.db")
        self.engine = create_engine('sqlite:///'+self._dbfile)
        self.storage = Storage(self.engine)
        self.metadata = MetaData(bind=self.engine, reflect=True)

    def tearDown(self):
        os.remove(self._dbfile)
        pass

class TestSQLiteStorageTables(StorageTest, StorageTestTables, unittest.TestCase):

    def dummy(self):
        pass


class TestSQLiteStorageMethods(StorageTest, StorageTestMethods, unittest.TestCase):

    def dummy(self):
        pass



try:
    from builtins import object
except ImportError:
    pass


class ProseEngine(object):

    def __init__(self, app=None, storage=None):
        self.app = app
        self.storage = storage
        self.viewmethod_decorators()
        if self.app is not None and self.storage is not None:
            self.init_app(app, storage)

    def init_app(self, app, storage):
        self.app = app
        self.storage = storage
        self.config = self.app.config

        from .views import create_blueprint
        prose_app = create_blueprint(__name__, self)
        # extenral urls
        prefix = self.config.get("PROSE_URL_PREFIX", '/v1')

        self.app.register_blueprint(prose_app, url_prefix=prefix)

        self.app.extensions["FLASK_PROSE_ENGINE"] = self  # duplicate
        self.app.extensions["prose"] = self

    def viewmethod_decorators(self, prose=(), corpora=()):
        """
        these methods are applied in views.create_blueprint

        viewmethod decorators can be appended to the
        MethodView class's decorator.  These wrapper functions
        can facilitate custom things like security.

        they are applied to the class not instance.
        """
        self.prose_api_decorators = prose
        self.corpora_api_decorators = corpora



from setuptools import setup, find_packages
import os
import re
import ast


def get_requirements(suffix=''):
    BASE_PATH = os.path.join(os.path.dirname(__file__), 'requirements')
    with open(os.path.join(BASE_PATH, 'requirements%s.txt' % suffix)) as f:
        rv = f.read().splitlines()
        return rv

def get_version(file):
    _version_re = re.compile(r'__version__\s+=\s+(.*)')

    with open(file, 'rb') as f:
        version = str(ast.literal_eval(_version_re.search(
            f.read().decode('utf-8')).group(1)))
        return version


setup(
    name='Flask-Prose',
    version=get_version('flask_prose/__init__.py'),
    url='https://github.com/slippers/Flask-Prose',
    license='MIT',
    author='Kirk Erickson',
    author_email='ekirk0@gmail.com',
    description='A flask extension for generating markov prose',
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    install_requires=get_requirements(),
    tests_require=['pytest'],
    test_suite='tests',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Framework :: Flask',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)



from flask import (
    Blueprint,
    abort,
    current_app,
    request,
    redirect,
    url_for,
    make_response,
    render_template
)
from flask_jsontools import (
    jsonapi,
    MethodView,
    methodview,
    SqlAlchemyResponse
)
from sqlalchemy import inspect


def _get_engine(app):
    # use flask.current_app
    return app.extensions["FLASK_PROSE_ENGINE"]


class ProseAPI(MethodView):

    decorators = (jsonapi, )

    @methodview(methods=('GET',))
    def get(self, uuid=None):
        engine = _get_engine(current_app)
        prose = engine.storage.prose(uuid=uuid)
        return SqlAlchemyResponse(prose)


    @methodview(methods=('POST',))
    def post(self):
        prose_id = request.form.get('prose_id')
        reaction = request.form.get('reaction')
        engine = _get_engine(current_app)
        return SqlAlchemyResponse(engine.storage.grock(prose_id, reaction))


class CorporaAPI(MethodView):

    decorators = (jsonapi, )

    @methodview(methods=('GET',))
    def get(self, uuid=None):
        engine = _get_engine(current_app)
        return SqlAlchemyResponse(engine.storage.corpora_list(uuid))

    @methodview(methods=('DELETE',), ifset=('uuid',))
    def delete(self, uuid):
        engine = _get_engine(current_app)
        return engine.storage.corpora_delete(uuid)

    @methodview(methods=('POST',))
    def post(self):
        engine = _get_engine(current_app)
        text = request.files['file'].read()
        label = request.form.get('label')
        source = request.form.get('source')
        corpora_id = engine.storage.corpora_save(label=label,
                                                 source=source,
                                                 text=text.decode('utf-8'))
        return SqlAlchemyResponse(engine.storage.corpora_list(uuid=corpora_id))


def create_blueprint(import_name, engine):

    prose_app = Blueprint('prose',
                          import_name,
                          template_folder='templates')

    # append custom api decorators
    ProseAPI.decorators = ProseAPI.decorators \
            + engine.prose_api_decorators

    # create the view
    prose_view = ProseAPI.as_view('prose_api')

    # rule
    prose_app.add_url_rule("/prose",
                           view_func=prose_view)

    prose_app.add_url_rule("/prose/<uuid:uuid>",
                           view_func=prose_view)

    CorporaAPI.decorators = CorporaAPI.decorators \
            + engine.corpora_api_decorators

    corpora_view = CorporaAPI.as_view('corpora_api')

    prose_app.add_url_rule('/corpora/<uuid:uuid>',
                           view_func=corpora_view)

    prose_app.add_url_rule('/corpora',
                           view_func=corpora_view)


    return prose_app



import unittest
import os
from flask_prose import (
ProseMakerText,
ProseMakerSen
)


class TestProseMaker(unittest.TestCase):

    def setUp(self):
        xray = os.path.join(os.path.dirname(__file__), 'shatit')
        with open(xray, 'r') as myfile:
            data = myfile.read()
            self.pm = ProseMakerText(text=data)

    def test_sentences(self):
        sentences = self.pm.get_sentences()
        print(sentences[1])
        self.assertIsNotNone(sentences)

    def test_stanza(self):
        stanza = self.pm.stanza()
        print(stanza)
        self.assertIsNotNone(stanza)
        self.assertEquals(len(stanza), 5)

    def test_haiku(self):
        haiku = self.pm.haiku()
        print(haiku)
        self.assertEquals(len(haiku), 3)

    def test_ProseMakerSen(self):
        ps = ProseMakerSen(self.pm.sentences)
        self.assertListEqual(ps.sentences, self.pm.sentences)



import unittest
import os
import json
from flask import Flask, session
from flask_sqlalchemy import SQLAlchemy
from flask_prose import FSQLAStorage, ProseEngine
from flask_jsontools import FlaskJsonClient, JsonResponse
from functools import wraps


def wrapped_view(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        print('wrapped_view', args)
        return f(*args, **kwargs)
    return wrapper


class ApiTest(unittest.TestCase):

    def setUp(self):
        self.app = app = Flask(__name__)
        self.app.test_client_class = FlaskJsonClient
        self.app.debug = self.app.testing = True
        self.app.config['SECRET_KEY'] = os.urandom(24)

        #config sqlalchemy
        app.config['SQLALCHEMY_TRACK_MODIFICATIONS'] = False
        app.config['DATABASE_URI'] = 'sqlite://'
        app.config['SQLALCHEMY_BINDS'] = {'prose':'sqlite://'}
        self.db = db = SQLAlchemy(app)

        # config flask-prose
        self.storage = FSQLAStorage(db=db, bind_key='prose')
        self.prose = ProseEngine()
        self.prose.viewmethod_decorators(corpora=(wrapped_view,))
        self.prose.init_app(app=app, storage=self.storage)

        #print(app.url_map)

        self.xray = os.path.join(os.path.dirname(__file__), 'little_shak')

        with open(self.xray, 'r') as myfile:
            text = myfile.read()
            self.storage.corpora_save(label='setup1', text=text)
            self.storage.corpora_save(label='setup2', text=text)


        @app.before_first_request
        def create_user():
            self.db.create_all()

    def test_upload(self):
        with open(self.xray, 'rb') as myfile:
            with self.app.test_client() as c:
                rv = c.post('/v1/corpora',
                            data = {'label':'test_upload', 'file': (myfile,'shatit_now')})
                self.assertEqual(rv.status_code, 200)
                self.assertIsInstance(rv, JsonResponse)


                print('get_json', type(rv.get_json()), rv.get_json())
                rv_dict = json.loads(rv.get_json())
                print('rv_dict', type(rv_dict), rv_dict)

                rl = c.get('/v1/corpora/{0}'.format(rv_dict[0]['id']))
                self.assertEqual(rl.status_code, 200)
                self.assertIsInstance(rl, JsonResponse)
                print(str(rl.get_json()))

                rp = c.get('/v1/prose')
                print(rp.get_json())

    def test_corpora_list(self):
        with self.app.test_client() as c:
            # get all corpora
            rv = c.get('/v1/corpora')
            corpora = json.loads(rv.get_json())
            print(rv.get_json(), corpora)
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)
            self.assertGreaterEqual(len(corpora), 1)

            # get one corpora by id
            rv = c.get('/v1/corpora/' + corpora[0]['id'])
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)

    def test_prose(self):
         with self.app.test_client() as c:
            # get a prose  = random
            rv = c.get('/v1/prose')
            prose = json.loads(rv.get_json())
            print('@@@', type(prose), prose)
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)

            # get the prose by id
            rv = c.get('/v1/prose/' + prose['id'])
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)

    def test_grock(self):
        with self.app.test_client() as c:
            # get a prose
            rv = c.get('/v1/prose')
            prose = json.loads(rv.get_json())

            print('####',type(rv.get_json()), type(prose), prose)
            rv = c.post('/v1/prose', data={'prose_id':prose['id'], 'reaction':'omg'})
            self.assertEqual(rv.status_code, 200)
            self.assertIsInstance(rv, JsonResponse)
